<?php

namespace LWS\WOOREWARDS\Wizards;

// don't call the file directly
if (!defined('ABSPATH')) exit();

/** satic class to manage activation and version updates. */
class Standard extends \LWS\WOOREWARDS\Wizards\Subwizard
{
	function getHierarchy()
	{
		// first step must be named 'ini' as the faky one from wizard.php
		return array(
			'ini',
			'met',
			'rew',
			'sum',
		);
	}

	function getStepTitle($slug)
	{
		switch ($slug) {
			case 'ini':
				return __("Standard System", LWS_WOOREWARDS_DOMAIN);
			case 'met':
				return __("Methods to earn points", LWS_WOOREWARDS_DOMAIN);
			case 'rew':
				return __("Reward", LWS_WOOREWARDS_DOMAIN);
			case 'sum':
				return __("Summary", LWS_WOOREWARDS_DOMAIN);
		}
		return $slug;
	}

	function getPage($slug, $mode = '')
	{
		switch ($slug) {
			case 'ini':
				return array(
					'title' => $this->getStepTitle($slug),
					'help'  => __("Welcome to this Wizard. This tool will help you configure your first loyalty system in less than 5 minutes.", LWS_WOOREWARDS_DOMAIN) . "<br/>" .
						__("This wizard is limited for WooRewards Standard.", LWS_WOOREWARDS_DOMAIN) . "<br/>" .
						__("If you want to have access to all methods and rewards, please consider trying WooRewards Pro.", LWS_WOOREWARDS_DOMAIN),
					'groups' => array(
						array(
							'fields'  => array(
								array(
									'id'    => 'system_title',
									'title' => __('Loyalty system name', LWS_WOOREWARDS_DOMAIN),
									'type'  => 'text',
									'extra' => array(
										'placeholder' => __('Standard System', LWS_WOOREWARDS_DOMAIN),
										'help' => __("Name your loyalty system. If you leave it empty, it will be named automatically", LWS_WOOREWARDS_DOMAIN),
									),
								),
							)
						)
					)
				);
			case 'met':
				return array(
					'title' => $this->getStepTitle($slug),
					'help'  => __("Your customers will earn points every time they perform the actions defined here.", LWS_WOOREWARDS_DOMAIN) . "<br/>" .
						__("Set points for all the methods you want and ignore the ones you don't want. You can change all these settings later.", LWS_WOOREWARDS_DOMAIN) . "</br>" .
						__("Some categories are not accessible in the free version of WooRewards.", LWS_WOOREWARDS_DOMAIN),
					'groups' => array(
						array(
							'fields' => array(
								array(
									'id'    => 'category',
									'title' => __("Select a Category", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'radiogrid', // radiogrid is specific to the wizard
									'extra' => array(
										'source' => array(
											array('value' => 'orders', 'icon' => 'lws-icon lws-icon-shop', 'label' => __("Orders", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'product', 'icon' => 'lws-icon lws-icon-barcode', 'class' => 'inactive', 'label' => __("Products", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'rate', 'icon' => 'lws-icon lws-icon-star-full', 'class' => 'inactive', 'label' => __("Rate us !", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'social', 'icon' => 'lws-icon lws-icon-b-meeting', 'class' => 'inactive', 'label' => __("Talk about us !", LWS_WOOREWARDS_DOMAIN)),
										),
										'default' => 'orders',
									),
								)
							),
						),
						array(
							'require' => array('selector' => 'input#category', 'value' => 'orders'),
							'fields'  => array(
								array(
									'id'    => 'spent_earn',
									'title' => sprintf(_x("Points for each %s spent", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?'),
									'type'  => 'text',
									'extra' => array(
										//'pattern' => "\\d*",
										'placeholder' => __('Number | Empty to ignore', LWS_WOOREWARDS_DOMAIN),
									),
								),
								array(
									'id'    => 'order_earn',
									'title' => sprintf(_x("Points on order placed", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?'),
									'type'  => 'text',
									'extra' => array(
										//'pattern' => "\\d*",
										'placeholder' => __('Number | Empty to ignore', LWS_WOOREWARDS_DOMAIN),
									),
								),
								array(
									'id'    => 'first_order_earn',
									'title' => sprintf(_x("Extra points on first order", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?'),
									'type'  => 'text',
									'extra' => array(
										//'pattern' => "\\d*",
										'placeholder' => __('Number | Empty to ignore', LWS_WOOREWARDS_DOMAIN),
									),
								),
							),
						),
					),
				);
			case 'rew':
				return array(
					'title' => $this->getStepTitle($slug),
					'help'  => __("Select the reward for your customers. The rewards types are limited in the standard version. You can change these settings later.", LWS_WOOREWARDS_DOMAIN),
					'groups' => array(
						array(
							'fields' => array(
								array(
									'id'    => 'reward',
									'title' => __("Select a reward", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'radiogrid', // radiogrid is specific to the wizard
									'extra' => array(
										'source' => array(
											array('value' => 'coupon'							,'icon' => 'lws-icon lws-icon-coins'	,'label' => sprintf(_x("Coupon (%s)", "Coupon Unlockable", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?')),
											array('value' => 'discount'							,'icon' => 'lws-icon lws-icon-discount'	,'label' => __("Discount (%)", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'product'	,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-gift'		,'label' => __("Free Product", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'shipping'	,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-supply'	,'label' => __("Free Shipping", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'variable'	,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-discount'	,'label' => __("Variable Discount", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'badgge'	,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-reward'	,'label' => __("Badge", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'role'		,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-users'	,'label' => __("User Role", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'role'		,'class' => 'inactive'	,'icon' => 'lws-icon lws-icon-crown'	,'label' => __("VIP Membership", LWS_WOOREWARDS_DOMAIN)),
										),
										'default' => 'coupon',
									),
								)
							),
						),
						array(
							'fields' => array(
								array(
									'id'    => 'needed',
									'title' => __("Points Needed", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'text',
								),
							),
						),
						array(
							'require' => array('selector' => 'input#reward', 'value' => 'coupon'),
							'fields' => array(
								array(
									'id'    => 'coupon_amount',
									'title' => sprintf(_x("Coupon Amount (%s)", "Coupon Unlockable", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?'),
									'type'  => 'text',
									'extra' => array(
										'placeholder' => '1',
										'help' => __("Every time customers have enough points, those points will be used to generate a coupon of that amount.", LWS_WOOREWARDS_DOMAIN),
									),
								),
							),
						),
						array(
							'require' => array('selector' => 'input#reward', 'value' => 'discount'),
							'fields' => array(
								array(
									'id'    => 'discount_amount',
									'title' => __("Discount (%)", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'text',
									'extra' => array(
										'placeholder' => '1',
										'help' => __("Every time customers have enough points, those points will be used to generate a discount coupon with that percentage.", LWS_WOOREWARDS_DOMAIN),
									),
								),
							),
						),
					),
				);
			case 'sum':
				return array(
					'title' => $this->getStepTitle($slug),
					'help'  => __("You're almost done. Check your settings below and submit if you're satisfied with the settings.", LWS_WOOREWARDS_DOMAIN),
					'groups' => array(
						array(
							'fields' => array(
								array(
									'id'    => 'summary',
									'title' => __("Settings Summary", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'custom', // radiogrid is specific to the wizard
									'extra' => array(
										'content' => $this->getSummary(),
										'help' => __("Do you want to start your loyalty system at the end of this wizard ? If you select No, you'll have to start it manually later.", LWS_WOOREWARDS_DOMAIN),
									),
								),
								array(
									'id'    => 'start',
									'title' => __("Start the program ?", LWS_WOOREWARDS_DOMAIN),
									'type'  => 'radiogrid', // radiogrid is specific to the wizard
									'extra' => array(
										'source' => array(
											array('value' => 'yes', 'label' => __("Yes", LWS_WOOREWARDS_DOMAIN)),
											array('value' => 'no', 'label' => __("No", LWS_WOOREWARDS_DOMAIN)),
										),
										'default' => 'yes',
										'help' => __("Do you want to start your loyalty system at the end of this wizard ? If you select No, you'll have to start it manually later.", LWS_WOOREWARDS_DOMAIN),
									),
								)
							),
						),
					)
				);
			default:
				return array();
		}
	}

	function getActiveStatus($tested = '')
	{
		$data = $this->getData();
		$exists = false;
		$methods = $this->getDataValue($data, 'met', false, $exists);
		foreach ($methods as $method) {
			if ($method['order_methods'] == $tested)
				return 'inactive';
		}
		return ('');
	}

	function getSummary()
	{
		$data = $this->getData();
		$exists = false;
		$currency = \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?';
		$summary = "<div class='lws-wizard-summary-container'>";
		/* Loyalty system name */
		$usedData = $this->getDataValue($data,'ini',false,$exists);
		$system = reset($usedData);
		$summary .= "<div class='lws-wizard-summary-title'>".__("Loyalty System", LWS_WOOREWARDS_DOMAIN)."</div>";
		$value = ($system['system_title']) ? $system['system_title'] : __("Standard System", LWS_WOOREWARDS_DOMAIN);
		$summary .= "<div class='lws-wizard-summary-label'>".__("Loyalty System Name", LWS_WOOREWARDS_DOMAIN)."</div>";
		$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";

		/* Earning methods */
		$usedData =$this->getDataValue($data,'met',false,$exists);
		$methods = reset($usedData);
		$summary .= "<div class='lws-wizard-summary-title'>".__("Methods to earn points", LWS_WOOREWARDS_DOMAIN)."</div>";
		if($methods['spent_earn'] && $methods['spent_earn']>0){
			$value = sprintf(__(' %s points earned for each %s spent', LWS_WOOREWARDS_DOMAIN),$methods['spent_earn'],$currency);
			$summary .= "<div class='lws-wizard-summary-label'>".__("Spend Money", LWS_WOOREWARDS_DOMAIN)."</div>";
			$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";
		}
		if($methods['order_earn'] && $methods['order_earn']>0){
			$value = sprintf(__(' %s points for each placed order', LWS_WOOREWARDS_DOMAIN),$methods['order_earn']);
			$summary .= "<div class='lws-wizard-summary-label'>".__("Place an order", LWS_WOOREWARDS_DOMAIN)."</div>";
			$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";
		}
		if($methods['first_order_earn'] && $methods['first_order_earn']>0){
			$value = sprintf(__(' %s extra points for the first order', LWS_WOOREWARDS_DOMAIN),$methods['first_order_earn']);
			$summary .= "<div class='lws-wizard-summary-label'>".__("Place a first order", LWS_WOOREWARDS_DOMAIN)."</div>";
			$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";
		}
		/* Rewards */
		$usedData =$this->getDataValue($data,'rew',false,$exists);
		$rewards = reset($usedData);
		$summary .= "<div class='lws-wizard-summary-title'>".__("Reward", LWS_WOOREWARDS_DOMAIN)."</div>";
		if($rewards['reward']=="discount"){
			$value = sprintf(__(' %s percent discount for %s points', LWS_WOOREWARDS_DOMAIN),$rewards['discount_amount'],$rewards['needed']);
			$summary .= "<div class='lws-wizard-summary-label'>".__("Percentage Discount", LWS_WOOREWARDS_DOMAIN)."</div>";
			$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";
		}
		if($rewards['reward']=="coupon"){
			$value = sprintf(__(' %s%s discount for %s points', LWS_WOOREWARDS_DOMAIN),$rewards['coupon_amount'],$currency, $rewards['needed']);
			$summary .= "<div class='lws-wizard-summary-label'>".__("Fixed Discount", LWS_WOOREWARDS_DOMAIN)."</div>";
			$summary .= "<div class='lws-wizard-summary-value'>{$value}</div>";
		}

		$summary .= "</div>";
		return ($summary);
	}

	function isValid($step, &$submit)
	{
		$err = array();
		if( $step == 'met' )
		{
			if( !$this->isIntGE0($submit, 'spent_earn') )
				$err[] = sprintf(__("Points for each %s spent expects numeric value greater than zero or leave blank.", LWS_WOOREWARDS_DOMAIN), \LWS_WooRewards::isWC() ? \get_woocommerce_currency_symbol() : '?');

			if( !$this->isIntGE0($submit, 'order_earn') )
				$err[] = __("Points on order placed expects numeric value greater than zero or leave blank.", LWS_WOOREWARDS_DOMAIN);

			if( !$this->isIntGE0($submit, 'first_order_earn') )
				$err[] = __("Extra points on first order expects numeric value greater than zero or leave blank.", LWS_WOOREWARDS_DOMAIN);
		}
		else if( $step == 'rew' )
		{
			$pts = isset($submit['needed']) ? trim($submit['needed']) : '';
			if( intval($pts) <= 0 )
			{
				$err[] = __("Points Needed expects numeric value greater than zero.", LWS_WOOREWARDS_DOMAIN);
			}
			$rew = isset($submit['reward']) ? trim($submit['reward']) : '';
			if( $rew == 'discount' )
			{
				if( !$this->isFloatInRangeEI($submit, 'discount_amount', 0.0, 100.0) )
					$err[] = __("Please, set a reward percentage greater than 0% up to 100%.", LWS_WOOREWARDS_PRO_DOMAIN);
			}
			else if( $rew == 'coupon' )
			{
				if( !$this->isFloatGT0($submit, 'coupon_amount') )
					$err[] = __("Please, set a positive reward amount.", LWS_WOOREWARDS_PRO_DOMAIN);
			}
			else
				$err[] = __("Please, select a reward type.", LWS_WOOREWARDS_DOMAIN);
		}
		return $err ? $err : true;
	}

	/** Instanciate pools, events, unlockables, etc. */
	function submit(&$data)
	{
		if( !isset($data['data']) )
			return false;
		$pool = $this->getDefaultPool();
		$pool->setOptions(array(
			'type'      => \LWS\WOOREWARDS\Core\Pool::T_STANDARD,
			'public'    => 'yes' === $this->getValue($data['data'], 'start', 'sum/*'),
			'title'     => $this->getValue($data['data'], 'system_title', 'ini/*', __("Standard System", LWS_WOOREWARDS_DOMAIN)),
			'whitelist' => array(\LWS\WOOREWARDS\Core\Pool::T_STANDARD),
		));

		$category = $this->getValue($data['data'], 'category', 'met/*');
		if( 'orders' === $category )
		{
			$this->deleteEvents($pool, 'lws_woorewards_events_orderamount');
			$this->deleteEvents($pool, 'lws_woorewards_events_ordercompleted');
			$this->deleteEvents($pool, 'lws_woorewards_events_firstorder');

			$pool->addEvent(new \LWS\WOOREWARDS\Events\OrderAmount(),    \absint($this->getValue($data['data'], 'spent_earn', 'met/*', 0)));
			$pool->addEvent(new \LWS\WOOREWARDS\Events\OrderCompleted(), \absint($this->getValue($data['data'], 'order_earn', 'met/*', 0)));
			$pool->addEvent(new \LWS\WOOREWARDS\Events\FirstOrder(),     \absint($this->getValue($data['data'], 'first_order_earn', 'met/*', 0)));
		}

		$this->deleteUnlockables($pool, 'lws_woorewards_unlockables_coupon');
		$coupon = new \LWS\WOOREWARDS\Unlockables\Coupon();

		$reward = $this->getValue($data['data'], 'reward', 'rew/*');
		if( 'coupon' === $reward )
		{
			$coupon->setInPercent(false);
			$coupon->setValue($this->getValue($data['data'], 'coupon_amount', 'rew/*', 0));
		}
		else if( 'discount' === $reward )
		{
			$coupon->setInPercent(true);
			$coupon->setValue($this->getValue($data['data'], 'discount_amount', 'rew/*', 0));
		}
		$pool->addUnlockable($coupon, \absint($this->getValue($data['data'], 'needed', 'rew/*', 0)));

		$pool->save();
		if( !$pool->getId() )
			return false;
		else
		{
			// set default pool marks
			\clean_post_cache($pool->getId());
			\update_post_meta($pool->getId(), 'wre_pool_prefab', 'yes');
			\update_option('lws_wr_default_pool_name', $pool->getName());
			return \add_query_arg('page', LWS_WOOREWARDS_PAGE.'.loyalty', \admin_url('admin.php'));
		}
	}

	function deleteEvents(&$pool, $type)
	{
		$e = $pool->getEvents()->filter(function($item)use($type){return $item->getType() == $type;});
		while( $e->count() )
		{
			$item = $e->last();
			$e->remove($item);
			$pool->removeEvent($item);
			$item->delete();
		}
	}

	function deleteUnlockables(&$pool, $type)
	{
		$u = $pool->getUnlockables()->filter(function($item)use($type){return $item->getType() == $type;});
		while( $u->count() )
		{
			$item = $u->last();
			$u->remove($item);
			$pool->removeUnlockable($item);
			$item->delete();
		}
	}

	function getDefaultPool($deep=true)
	{
		/// In free version, it can be only one.
		$pools = \LWS\WOOREWARDS\Collections\Pools::instanciate()->load(array(
			'numberposts' => 1,
			'meta_query'  => array(
				array(
					'key'     => 'wre_pool_prefab',
					'value'   => 'yes', // This cannot be empty because of a bug in WordPress
					'compare' => 'LIKE'
				),
				array(
					'key'     => 'wre_pool_type',
					'value'   => \LWS\WOOREWARDS\Core\Pool::T_STANDARD,
					'compare' => 'LIKE'
				)
			),
			'deep' => $deep
		));

		if( $pools->count() <= 0 )
		{
			$pools = \LWS\WOOREWARDS\Collections\Pools::instanciate()->load();
			$name = 'default';
			if( \is_multisite() )
				$name .= \get_current_blog_id();
			return $pools->create($name)->last();
		}
		else
		{
			return $pools->last();
		}
	}
}
